package com.sd.wln.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.after.po.AfterSaleOrder;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.constant.BomClassifyConstant;
import com.sd.business.entity.order.enums.OrderClassifyEnum;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.service.after.AfterSaleOrderService;
import com.sd.business.service.order.*;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.util.CodeEnum;
import com.sd.wln.service.WlnReturnOrderService;
import com.sd.wln.service.WlnSalesReturnStockInService;
import com.sd.wln.util.WlnUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@Service
public class WLnSalesReturnStockInServiceImpl implements WlnSalesReturnStockInService {


    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private OrderExchangeService orderExchangeService;

    @Autowired
    private OrderExchangeDetailService orderExchangeDetailService;

    @Autowired
    private AfterSaleOrderService afterSaleOrderService;

    @Autowired
    private WlnReturnOrderService wlnReturnOrderService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncSalesReturnStockIn() {
        // 同步售后单明细
        wlnReturnOrderService.syncReturnOrder();

        // 获取万里牛销售退货入库明细
        List<JSONObject> list = getWlnSalesReturnStockIn();

        List<AfterSaleOrder> afterSaleOrderList = afterSaleOrderService.list(q -> q.eq(AfterSaleOrder::getStatus, StatusConstant.NO));

        // 获取订单数据
        List<String> orderWlnCodeList = afterSaleOrderList.stream().map(AfterSaleOrder::getOrderWlnCode).collect(Collectors.toList());
        Map<String, OrderInfo> orderInfoMap = orderService.mapKEntity(OrderInfo::getWlnCode, q -> q.in(OrderInfo::getWlnCode, orderWlnCodeList));
        List<Long> orderIds = orderInfoMap.values().stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<OrderSku>> orderSkuMap = orderSkuService.mapKGroup(OrderSku::getOrderId, q -> q.in(OrderSku::getOrderId, orderIds));
        Map<Long, List<OrderSkuBom>> orderSkuBomMap = orderSkuBomService.mapKGroup(OrderSkuBom::getOrderSkuId, q -> q.in(OrderSkuBom::getOrderId, orderIds));
        List<Long> bomSpecIdList = orderSkuBomMap.values().stream()
                .flatMap(item -> item.stream().map(OrderSkuBom::getBomSpecId))
                .distinct()
                .collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        // 获取sku信息
        List<String> skuSpecUidList = afterSaleOrderList.stream().map(AfterSaleOrder::getWlnSkuSpecId).collect(Collectors.toList());
        Map<String, SkuSpec> skuSpecMap = skuSpecService.mapKEntity(SkuSpec::getWlnUid, q -> q.in(SkuSpec::getWlnUid, skuSpecUidList));

        Map<String, List<AfterSaleOrder>> orderCodeMap = afterSaleOrderList.stream().collect(Collectors.groupingBy(AfterSaleOrder::getOrderCode));
        Map<String, List<AfterSaleOrder>> orderWlnCodeMap = afterSaleOrderList.stream().collect(Collectors.groupingBy(AfterSaleOrder::getOrderWlnCode));
        Map<String, List<AfterSaleOrder>> codeMap = afterSaleOrderList.stream().collect(Collectors.groupingBy(AfterSaleOrder::getCode));

        List<OrderExchange> orderExchangeList = new ArrayList<>();
        List<OrderExchangeDetail> orderExchangeDetailList = new ArrayList<>();
        List<Long> editAfterSaleOrderIdList = new ArrayList<>();

        for (JSONObject item : list) {
            if (!Objects.equals(item.getString("storage_code"), "T007")) {
                continue;
            }

            // 获取万里牛售后单数据
            List<AfterSaleOrder> saleOrderList = new ArrayList<>();
            String fromExchangeNo = item.getString("from_exchange_no");
            if (StrUtil.isNotBlank(fromExchangeNo)) {
                saleOrderList = codeMap.getOrDefault(fromExchangeNo, Collections.emptyList());
            }
            // 订单号存在两种，XD和普通单号
            String orderCode = item.getString("tp_tid");
            if (saleOrderList.isEmpty()) {
                saleOrderList = orderCodeMap.getOrDefault(orderCode, Collections.emptyList());
            }
            if (saleOrderList.isEmpty()) {
                saleOrderList = orderWlnCodeMap.getOrDefault(orderCode, Collections.emptyList());
            }

            // 没有万里牛售后单时，不生成售后订单
            if (saleOrderList.isEmpty()) {
                continue;
            }

            Map<String, AfterSaleOrder> afterSaleOrderMap = saleOrderList.stream().collect(Collectors.toMap(AfterSaleOrder::getSkuSpecCode, Function.identity()));

            // 入库单明细
            List<JSONObject> detailsList = item.getJSONArray("details").toJavaList(JSONObject.class);


            // 出库单明细
            OrderExchange orderExchange = null;

            for (JSONObject itemDetails : detailsList) {
                AfterSaleOrder afterSaleOrder = afterSaleOrderMap.get(itemDetails.getString("sku_no"));
                if (afterSaleOrder == null) {
                    continue;
                }

                OrderInfo orderInfo = orderInfoMap.get(afterSaleOrder.getOrderWlnCode());
                // 无理由订单不处理
                if (orderInfo == null || Objects.equals(orderInfo.getClassify(), OrderClassifyEnum.NO_REASON_ORDER.getKey())) {
                    continue;
                }
                SkuSpec skuSpec = skuSpecMap.get(afterSaleOrder.getWlnSkuSpecId());
                if (skuSpec == null || Objects.equals(skuSpec.getGiftTag(), StatusConstant.YES)) {
                    continue;
                }
                OrderSku orderSku = orderSkuMap
                        .getOrDefault(orderInfo.getId(),Collections.emptyList())
                        .stream()
                        .filter(i -> Objects.equals(i.getSkuSpecId(), skuSpec.getId()))
                        .findAny()
                        .orElse(null);
                if (orderSku == null) {
                    continue;
                }
                // 包材成本只退可二次利用的包材
                List<OrderSkuBom> orderSkuBomList = orderSkuBomMap.getOrDefault(orderSku.getId(), Collections.emptyList());
                BigDecimal packagingMaterialCost = orderSkuBomList.stream()
                        .map(b -> {
                            BomSpecBo bomSpecBo = bomSpecBoMap.get(b.getBomSpecId());
                            if (bomSpecBo == null
                                    || !(Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.MESH_BAG)
                                    || Objects.equals(bomSpecBo.getClassifyId(), BomClassifyConstant.SUSPENDERS))) {
                                return BigDecimal.ZERO;
                            }
                            return b.getUnitPrice().multiply(b.getQuantity());
                        })
                        .reduce(BigDecimal.ZERO, BigDecimal::add);

                // 新增出库单数据
                if (orderExchange == null) {
                    orderExchange = new OrderExchange();
                    orderExchange.setId(IdWorker.getId());
                    orderExchange.setOrderInfoId(orderInfo.getId());
                    orderExchange.setCode(CodeEnum.TH_CODE.getCode());
                    orderExchange.setWarehouseId(WarehouseConstant.FINISHED_PRODUCT);
                    orderExchange.setType(StatusConstant.YES);
                    orderExchange.setStatus(StatusConstant.NO);
                    orderExchange.setReason(afterSaleOrder.getReason());
                    orderExchange.setRemark(afterSaleOrder.getRemark());
                    orderExchange.setCompletionTime(item.getDate("modify_time"));
                    orderExchange.setCompletionTimestamp(item.getLong("modify_time"));
                    orderExchangeList.add(orderExchange);
                }
                OrderExchangeDetail orderExchangeDetail = new OrderExchangeDetail();
                orderExchangeDetail.setOrderExchangeId(orderExchange.getId());
                orderExchangeDetail.setQuantity(afterSaleOrder.getQuantity());
                orderExchangeDetail.setReturnStatus(StatusConstant.NO);
                orderExchangeDetail.setExchangeStatus(StatusConstant.NO);
                orderExchangeDetail.setOrderSkuId(orderSku.getId());
                orderExchangeDetail.setCheckPassesQuantity(BigDecimal.ZERO);
                orderExchangeDetail.setReturnAmount((orderSku.getUnitPrice()
                        .add(packagingMaterialCost)).multiply(orderExchangeDetail.getQuantity()));
                orderExchangeDetailList.add(orderExchangeDetail);

                editAfterSaleOrderIdList.add(afterSaleOrder.getId());
            }
        }

        if (!editAfterSaleOrderIdList.isEmpty()) {
            // 更新万里牛售后单的状态
            afterSaleOrderService.update(q -> q.set(AfterSaleOrder::getStatus, StatusConstant.YES)
                    .in(BaseIdPo::getId, editAfterSaleOrderIdList));
        }
        // 新增售后订单
        orderExchangeService.saveBatch(orderExchangeList);
        orderExchangeDetailService.saveBatch(orderExchangeDetailList);
    }

    /**
     * 查询万里牛销售退货入库单数据
     */
    private List<JSONObject> getWlnSalesReturnStockIn() {
        OrderExchange orderExchange = orderExchangeService.getOne(q -> q
                .isNotNull(OrderExchange::getCompletionTimestamp)
                .orderByDesc(OrderExchange::getCompletionTimestamp));

        long startTime;
        long endTime = System.currentTimeMillis();
        if (orderExchange != null) {
            startTime = orderExchange.getCompletionTimestamp() + 1;
        } else {
            startTime = endTime - 1000 * 60 * 60 * 24 * 60L;
        }

        //
        startTime = DateUtil.parse("2023-12-01 00:00:00").getTime();

        //

        List<JSONObject> list = new ArrayList<>();

        while (startTime < endTime) {

            int page = 1;
            int size;
            do {
                try {
                    List<JSONObject> itemList = WlnUtil.getSalesReturnStockInList(page, 200, startTime, endTime);
                    page++;
                    size = itemList.size();
                    list.addAll(itemList);
                } catch (Exception e) {
                    log.error("销售退货入库单同步失败", e);
                    return Collections.emptyList();
                }
            } while (size >= 200);

            startTime = DateUtil.offsetDay(DateUtil.date(startTime), 7).getTime();
        }

        return list;
    }
}
